/**************************************************************************
 * @file        Init.c
 * @author      MDBU Software Team
 * @brief       Implementation of  drv83xx, MSP430F5529 MCU and AFE initialization.
 * @note        Copyright (c) 2016 Texas Instruments Incorporated.
 *              All rights reserved.
 ******************************************************************************/

#include "global.h"

// Controller
extern SENSORED_TRAP_Obj sensoredTrapController;
extern APPLICATION_STATUS applicationStatus;

// Host Controller
extern HOST_CONTROLLER_Obj HostController;
extern HOSTCONTROL_STATUS HostControl_Status;

void UART_Init(void);

/* function
 * Application_Init()
 * initializes the application structure
 * */
void Application_Init()
{
    applicationStatus.currentstate = SYSTEM_INIT;
    applicationStatus.previousstate = SYSTEM_INIT;
    applicationStatus.fault = NOFAULT;

    /*
     * One time initialization of the parameters for MDBU Serial
     *
     * */

    /* Configure Port 3.7 & Port 8.2 as Inputs to Identify the Device connected */
    P3DIR &= ~BIT7;
    P3REN &= ~BIT7;                             // Disable Pull-up, Pull-down
    P8DIR &= ~BIT2;
    P8REN &= ~BIT2;                             // Disable Pull-up, Pull-down
    /* Configure Port 7.4 as Input and check the Input logic;     */
    P7DIR &= ~BIT4;                            // Configure the GPIO as Input and check the status of the pin
    P7REN |= BIT4;                             // enable pull up/pull down of resistance
    P7OUT |= BIT4;                             // set pull up configuration.
    sensoredTrapController.DeviceVariant = DRV8343_DEVICE;  // drv83xx device family
    sensoredTrapController.ShuntVariant = 1;
    sensoredTrapController.SPIVariant = ((P3IN & BIT6)>>6);
    sensoredTrapController.DeviceID = sensoredTrapController.DeviceVariant;
    sensoredTrapController.DeviceID = ((sensoredTrapController.DeviceID<<1) + sensoredTrapController.ShuntVariant);
    sensoredTrapController.DeviceID = ((sensoredTrapController.DeviceID<<1) + sensoredTrapController.SPIVariant);
    mdbu_setTrgtInfo(PROD_ID,ALGO_ID,sensoredTrapController.DeviceID,
    		FW_VER_MAJ ,FW_VER_MIN,FW_VER_PATCH);

    /* initialized value of Duty cycle that should be reached for the selected potentiometer command setting.*/
    sensoredTrapController.TargetDutyCycle = 0;

    /* How many PWM cycles before increasing duty cycle. Initialized to every 100th PWM period.*/
    sensoredTrapController.Accelerationdivider = RAMP_RATE_DELAY;

    /* Low speed setting depending on application */
    sensoredTrapController.MINDutyCycle = MIN_DUTY_CYCLE;
    /* High speed setting depending on application //0x400 = 1024  --> 50% */
    sensoredTrapController.MAXDutyCycle = MAX_DUTY_CYCLE;
    /* Default shift required to read Hall state from GPIO pins */
    sensoredTrapController.ShiftForPINA = HALL_SHIFT_FOR_PIN_A;
    sensoredTrapController.ShiftForPINB = HALL_SHIFT_FOR_PIN_B;
    sensoredTrapController.ShiftForPINC = HALL_SHIFT_FOR_PIN_C;
    /* Sets the duty cycle during the calibration of Hall states*/
    sensoredTrapController.CALDutyCycle = CAL_DUTY_CYCLE;
    /* Defines number of PWM cycles Motor Hall Calibration is Executed */
    sensoredTrapController.HallCalibrationCycles = HALL_CALIB_CYCLES;
    /* Maximum allowed motor phase current*/
    sensoredTrapController.MotorPhaseCurrentLimit = MOTOR_PHASE_CURRENT_LIMIT;
    sensoredTrapController.HallAlignStartFlag = 0;
    sensoredTrapController.HallAlignDoneFlag = 0;

    /* Under/Over Voltage limits */
    sensoredTrapController.underVoltageLimit = ((UNDER_VOLTAGE_LIMIT * (float_t) 4095.0)/ FULL_SCALE_VOLTAGE);
    sensoredTrapController.overVoltageLimit = ((OVER_VOLTAGE_LIMIT * (float_t) 4095.0)/ FULL_SCALE_VOLTAGE);
    sensoredTrapController.MinPowerSupply = ((MIN_POWER_SUPPLY * (float_t)4095.0)/FULL_SCALE_VOLTAGE);
    /* Minimum Duty Cycle above which stall will be detected */
    sensoredTrapController.minStallDetectDuty = MIN_STALLDETECT_DUTY;

    /* Number of revolutions below which stall fault will be flagged */
    sensoredTrapController.stallDetectRevThreshold = STALLDETECT_REV_THRESHOLD;

    /* Time in milli seconds above which if motor doesnt spin min revolutions specified above(stallDetectRevThreshold) a stall fault is triggered */
    sensoredTrapController.stallDetectTimerThreshold = STALLDETECT_TIMER_THRESHOLD;

    /* Time in milli seconds after which system reinitialises itself if fault gets cleared */
    sensoredTrapController.autoFaultRecoveryTime = AUTO_FAULT_RECOVERY_TIME;

    /* This Variable holds the Ctrl type : 0 for 6PWM mode, 1 for 1 PWM mode*/
    sensoredTrapController.PWM_Mode = ONE_PWM_MODE;

    /* PWM Switching Frequency settings*/
    sensoredTrapController.PWMPeriod = PWM_PERIOD;   		/* This Variable holds the Motor parameter to set the PWM switching frequency as 25Mhz / PWM_PERIOD */
    sensoredTrapController.Counter_1M_Second = (25000 / sensoredTrapController.PWMPeriod);
    sensoredTrapController.Counter_1M_Second *= 1000;
    /* Initialize MDBU Serial physical layer */
    #ifdef MDBUSERIAL_USE_USB
      USB_setup(TRUE,TRUE);       /* MDBU Serial Protocol over USB */
    #else
      UART_Init();                /* MDBU Serial Protocol over UART */
    #endif

    /* Gate Drive Enable using Port 1.6 */
    P1DIR |= BIT6;
    P1OUT &= ~BIT6;

    DRV8x_Digital_Init();                                              				   /*Initialize MSP430F5529 peripherals */
    /* Analog initialization */
    DRV8x_Analog_Init();
}

/*function
 *  MotorVariabes_Init()
 * initializes the motor structure
 * */
void sensoredTrapController_Init()
{
    /* initialized value to reading from hall sensor (get hall effect pin state and shift to LSB) */
    sensoredTrapController.currentHallstate = 00;
    /* Set the direction flag value based on the Direction pin switch P1.1.*/
    sensoredTrapController.Direction_flag = FALSE;
    /* initialized value of Current DutyCycle 0% */
    sensoredTrapController.CurrentDutyCycle = 0x00;
    /* How fast increasing duty cycle per PWM cycle times acceleration divider initialized 1 */
    sensoredTrapController.RampRate = RAMP_RATE;
    /* Initialize a counter to compare with Acceleration Divider. Initialized to 0.*/
    sensoredTrapController.Accelerationcounter = 0x00;
    /*This keeps track of how many rotations (electrical) of the motor is completed since the last startup. */
    sensoredTrapController.RotationCount = 0x00;
    /*This is a counter used in Timer4 to check for the motor stall. */
    sensoredTrapController.StallDetectCounter = 0x00;
    /* This is the variable that is used to count the time from when a fault is cleared to when the
       motor starts again */
    sensoredTrapController.RestartDelay = 0x00;
    /* How much voltage applied for EVM */
    sensoredTrapController.VCCvoltage = 0x00;

    /* Read VCC voltage every 100ms */
    sensoredTrapController.readVccCounter = 0;

    /* This variable is used to measure the speed in timer counts of 16 bit resolution, Timer counts is inversely proportional to speed i.e. Maximum speed count represents minimum speed in frequency, 16 bit max value = 0xFFFFh */
    sensoredTrapController.SPDFdbk = 0xFFFF ;

    /* Interrupt counter to ensure the speed measurement with 16bit resolution*/
    sensoredTrapController.TimerOverflowFlag = 0;
}

void ADC_Init(void)
{
    ADC12CTL0 = ADC12ON + ADC12MSC + ADC12SHT0_0;       // Turn on ADC12, extend sampling time
                                                        // to avoid overflow of results
    ADC12CTL1 = ADC12SHP + ADC12CONSEQ_1;               // Use sampling timer, Single sequence
    ADC12MCTL0 = ADC12INCH_3;                           // ref+=AVcc, channel = A3 (Read the Vcc Voltage from drv83xxX )
    ADC12MCTL1 = ADC12INCH_4 + ADC12EOS;               // ref+=AVcc, channel = A4 (Read the Current Shunt amplifier reading from Phase A ) , End of Sequence
    ADC12CTL0 |= ADC12ENC;                              // Enable conversions
}

/* Function Clock Initialization to make use of 25MHZ setting DCO in Clock module of MSP4320F5529 */

void UCS_Init(void)
{
    WDTCTL = WDTPW + WDTHOLD;                       // Stop WDT
    SetVcoreUp (0x01);
    SetVcoreUp (0x02);
    SetVcoreUp (0x03);

    UCSCTL3 = SELREF_2;                             // Set DCO FLL reference = REFO
    UCSCTL4 |= SELA_2;                              // Set ACLK = REFO

    __bis_SR_register(SCG0);                        // Disable the FLL control loop
    UCSCTL0 = 0x0000;                               // Set lowest possible DCOx, MODx
    UCSCTL1 = DCORSEL_7;                            // Select DCO range 50MHz operation
    UCSCTL2 = FLLD_0 + 762;                         // Set DCO Multiplier for 25MHz
                                                    // (N + 1) * FLLRef = Fdco
                                                    // (762 + 1) * 32768 = 25MHz
                                                    // Set FLL Div = fDCOCLK/2
    __bic_SR_register(SCG0);                        // Enable the FLL control loop

    // Worst-case settling time for the DCO when the DCO range bits have been
    // changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
    // UG for optimization.
    // 32 x 32 x 25 MHz / 32,768 Hz ~ 780k MCLK cycles for DCO to settle
    __delay_cycles(782000);

    // Loop until XT1,XT2 & DCO stabilizes - In this case only DCO has to stabilize
    do
    {
        UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
        // Clear XT2,XT1,DCO fault flags
        SFRIFG1 &= ~OFIFG;                          // Clear fault flags
    }
    while(SFRIFG1 & OFIFG);                         // Test oscillator fault flag
}

/* Function Vcore to Use 25mhz DCO in clock module of MSP430F5529 use Vcore up settings to make DCO support 25Mhz*/
void SetVcoreUp(unsigned int level)
{
    // Open PMM registers for write
    PMMCTL0_H = PMMPW_H;
    // Set SVS/SVM high side new level
    SVSMHCTL = SVSHE + SVSHRVL0 * level + SVMHE + SVSMHRRL0 * level;
    // Set SVM low side to new level
    SVSMLCTL = SVSLE + SVMLE + SVSMLRRL0 * level;
    // Wait till SVM is settled
    while((PMMIFG & SVSMLDLYIFG) == 0)
    {
        ;
    }
    // Clear already set flags
    PMMIFG &= ~(SVMLVLRIFG + SVMLIFG);
    // Set VCore to new level
    PMMCTL0_L = PMMCOREV0 * level;
    // Wait till new level reached
    if((PMMIFG & SVMLIFG))
    {
        while((PMMIFG & SVMLVLRIFG) == 0)
        {
            ;
        }
    }
    // Set SVS/SVM low side to new level
    SVSMLCTL = SVSLE + SVSLRVL0 * level + SVMLE + SVSMLRRL0 * level;
    // Lock PMM registers for write access
    PMMCTL0_H = 0x00;
}

void GPIO_Init(void)
{

/* PWM Initialization Using Ports P1.3, P1.5 , P2.5 for A , B , C Phases High side , P1.2, P1.4 , P2.4 for A , B , C Phases Low side respectively */

    PWM_Init();   // Configure PWM pins appropriately based on PWM mode of operation

/* Configure Port 2.0 , 2.2, 2.6 as inputs to read hall state A,B,C phases respectively */
    P2DIR &= ~(BIT0 | BIT3 | BIT6);
    P2REN |= BIT0 | BIT3 | BIT6;
    P2OUT |= BIT0 | BIT3 | BIT6;
    //P2IE |= BIT0 | BIT3 | BIT6;
    P2IFG |= 0x00;
	P2IES |= (P2IN & (BIT0 | BIT3 | BIT6));     // Port Interrupts edges selected based on the Current position of Hall Sensors


/* Configure Port 2.7 as input for sensing faults and enable Interrupt */
    P2DIR &= ~BIT7;
    P2REN |= BIT7;                              // When a GPIO pin is configured as Input, Enable resistance to the pin by setting Px.REN and PX.OUT
    P2OUT |= BIT7;
    //P2IE |= BIT7;
    P2IFG |= 0x00;
    P2IES |= BIT7;

/* UART selection for host interface*/
    P3SEL = BIT3+BIT4;                        // P3.3,4 = USCI_A0 TXD/RXD
/* Configure the Mode pin for drv83xxH devices */
//    if(sensoredTrapController.PWM_Mode == SIX_PWM_MODE) // If 6x PWM mode configure the Mode pin to pull down
//    {
//        P4SEL &= ~BIT3;  // Select the GPIO functionality of the pin
//    	P4DIR |= BIT3;   // Set the pin as Output
//        P4OUT &= ~BIT3;  // Set the pin logic low
//    }
//    else // If 1x PWM mode configure mode pin to High Impedence state
//    {
//        P4SEL &= ~BIT3;	 // Select the GPIO functionality of the pin
//    	P4DIR &= ~BIT3;  // Set the pin as Input
//    	P4REN &= ~BIT3;  // Diable the the Pull up/ pull down set the pin to high impedence
//    }
}

/*function
 * UART_Init()
 * Initializes UART for host interface.
 * */
void UART_Init(void)
{
	UCA0CTL1 |= UCSWRST;                      // **Put state machine in reset**
	UCA0CTL1 |= UCSSEL_2;                     // SMCLK
	UCA0BR0 = 162 ;                           // 25MHz - 9600 Baud 25M/9600= 2604.1667 , 2604.1667/16 = 162.76 , 0.76 *16 = 12.16
	UCA0BR1 = 0;
	UCA0MCTL = UCBRS_0 + UCBRF_12 + UCOS16;   // Modln UCBRSx=0, UCBRFx=12,
	UCA0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**
	UCA0IE |= UCRXIE;                         // Enable USCI_A0 RX interrupt
}
/*function
 * PWM_Init()
 * Initializes PWM pins for different PWM modes.
 * */
void PWM_Init(void)
{
    if(sensoredTrapController.PWM_Mode == SIX_PWM_MODE)  // If six PWM mode set the INLA , INLB , INHB as outputs
    {
        P1DIR |= BIT2 | BIT3 | BIT4 | BIT5;          //  P1.2,P1.3,P1.4,P1.5 to output direction for driving PWM to drv83xx
        P2DIR |= BIT4 | BIT5;                        //  P2.4,P2.5 to output direction for driving PWM to drv83xx

        P1OUT &= ~(BIT2 | BIT3 | BIT4 | BIT5);
        P1SEL |= BIT2 | BIT3 | BIT4 | BIT5;          // Set P1.2,P1.3,P1.4,P1.5 for Generating PWM for B phase High side and C phase High side respectively
        P2OUT &= ~(BIT4 | BIT5);
        P2SEL |= BIT4 | BIT5;                        // Set P2.4,P2.5 for Generating PWM for A phase High side
    }
    else                            // If 1xPWM mode set appropriate pin configurations for Gate driver Pins INxx
    {
// Configure P1.2,P1.3 as outputs to set Direction and applying brakes
    	P1OUT |= BIT2;                  // Motor brakes are removed  by default
        if(sensoredTrapController.Direction == TRUE)
        {
            P1OUT |= BIT3;                  // Set the direction to high using INHC on drv83xx EVM
        }
        else
        {
            P1OUT &= ~BIT3;                  // Set the direction to low using INHC on drv83xx EVM
        }
        P1DIR |= BIT2 | BIT3 ;          //  P1.2,P1.3 to output direction for applying brakes & setting direction to drv83xx
// Set the 1 PWM pin on INHA
        P2OUT &= ~(BIT5);
        P2DIR |= BIT5;                     //  Set P2.5 to output direction for driving 1 PWM to drv83xx
        P2SEL |= BIT5;                     // Set P2.5 for Generating 1 PWM for A phase High side
// Configure INLA,INLB,INHB which are connected to Hall inputs
        P1DIR &= ~(BIT4 | BIT5) ;          //  Set P1.4 , P1.5 as Input pins which are directly connected to Hall B & C
        P2DIR &= ~BIT4;                    //  Set P2.4 as Input pin which is connected to Hall A
        P2REN &= ~BIT4 ;
        P1REN &= ~(BIT4 | BIT5);			   // Pull up the Input pins which are connected to Hall pins as High Impedence states
		//P2IE |= BIT0 | BIT2 | BIT6;
    }

}
/* USCI Initialization
   The USCI is initialized with the following setting
   3-pin, 8-bit SPI master,Clock polarity high,MSB
 */
void SPI_Init(void)
{
	/* SPI Ports Initialization * Port 3.0, 3.1, 3.2 is used for SIMO SOMI and SCLK respectively , Port 2.0 is Used for nSCS enable*/
    P3OUT &= ~(BIT0 | BIT2);
    P3DIR |= BIT0 | BIT2;                       /*  Set SIMO, CLK as outputs */
    P3DIR &= ~BIT1;                             /* SOMI, slave out master in defined as input P3.1 */
    P3SEL |= BIT0 | BIT1;                       // P3.0,3.1 option select for UCBOSIMO and UCBOSOMI
    P3SEL |= BIT2;                              // P3.2 option select for UCBOCLK function
    P2OUT |= BIT2;
    P2DIR |= BIT2;                              // Set P2.3 to output direction for nSCS

    UCB0CTL1 |= UCSWRST;                          // **Put state machine in reset**
    UCB0CTL0 |= UCMST + UCSYNC + UCMSB;           // 3-pin, 8-bit SPI master
    // Clock polarity high, MSB
    UCB0CTL1 |= UCSSEL_3;                         // MCLK
    UCB0BR0 = 10;                                 // Master Clock divided by 10 used by USCI clk
    UCB0BR1 = 0;                                  //
    UCB0CTL1 &= ~UCSWRST;                         // **Initialize USCI state machine**

    drv83xxSPISet();                              // make nSCS pin of drv83xx low to start communication with master SPI;
    drv83xxSPIReset();                            // make nSCS pin of drv83xx High to stop communication with master SPI;
    SPIDelay();
}

/*function
 * TIMERB_Init()
 * Initializes the TimerB interrupt to Support Fault recovery time and Stall fault detection
 * */
void TIMERB_Init(void)
{
    TBCTL = TBSSEL_2 | TBCLR | MC_1;            /* set continuous up count mode */
    TBCCR0 = sensoredTrapController.Counter_1M_Second;                    /*  1 ms period for compare 0 */
    TBCCTL0 |= CCIE;                            /* enable TimerB Period match interrupt */
    TB0R = 0x0000;                              /* reset counter */
}

/*function
 * TimerA0_Init()
 * initializes the TimerA0 to generate PWM pulses from TA0.1, TA0.2, TA0.3, TA0.4  also generate Period Interrupt for State machine
 * */
void TimerA0_Init(void)
{
    TA0CTL = TASSEL_2 | TACLR | MC_1;           	/* set SMCLK to run timer, Clear the Timer , continuous up count mode   */
    TA0CCR0 = (sensoredTrapController.PWMPeriod - 1);     /* set Period value as PWM_period*/

    TA0CCTL0 |= CCIE;                           /* Enable TimerA0 period match interrupt for State machine */
    TA0R = 0x0000;                              /* reset counter */

    /*set out put mode to GPIO low */
    TA0CCTL1 = OUTMOD_0;
    TA0CCTL2 = OUTMOD_0;
    TA0CCTL3 = OUTMOD_0;
    TA0CCTL4 = OUTMOD_0;

    /* reset compare */
    TA0CCR1 = 0x00;
    TA0CCR2 = 0x00;
    TA0CCR3 = 0x00;
    TA0CCR4 = 0x00;

    TA0CCTL1 = OUTMOD_2;
    TA0CCTL2 = OUTMOD_6;
    TA0CCTL3 = OUTMOD_2;
    TA0CCTL4 = OUTMOD_6;
}

/*function
 * TIMER_SPD_Init()
 * initializes TimerA1 for electrical speed measurement
 * */
void TIMER_SPD_Init()
{
	TA1CTL = TASSEL_2 | TACLR | MC_1 | ID_3 ;	/* set continues UP count mode , Clk Div / 2 */
	TA1CCR0 = 0x0FFFF;                      	/* This sets  Timer to count for 65535 counts at 25Mhz */
	TA1EX0 = TAIDEX_4;					 		          /* Divide clock by 5 , to make overall clock run at 25Mhz/(2*5) = 25MHz */
	TA1CCTL0 |= CCIE;					 		            /* enable TimerA1 period match interrupt */
	TA1R = 0x0000;						 		            /* reset counter */
}

/*function
 * TimerA2_Init()
 * initializes the TimerA2 to generate PWM pulses from TA2.2
 * */
void TimerA2_Init(void)
{
    TA2CTL = TASSEL_2 | TACLR | MC_1;           /* set SMCLK to run timer, Clear the Timer , continuous up count mode   */
    TA2CCR0 = (sensoredTrapController.PWMPeriod - 1); /* set Period value as PWM_period*/

    TA2R = 0x0000;                              /* reset Timer 2 counter */
    TA0R = TA2R;                          		/* reset Timer 0 counter to TA2R to sync the timers as PWM's were generated by both the timers*/

    /*set out put mode to GPIO low */
    TA2CCTL1 = OUTMOD_0;
    TA2CCTL2 = OUTMOD_0;

    /* reset compare */
    TA2CCR1 = 0x00;
    TA2CCR2 = 0x00;
    TA2CCTL1 = OUTMOD_2;                                                                                /* Select Output mode as Toggle/ReSet for all the switches*/
    TA2CCTL2 = OUTMOD_6;
}

/*function
 * DRV8x_Digital_Init(void)
 * initializes the MSP430F5529 IP's
 * */
void DRV8x_Digital_Init(void)
{
    UCS_Init();                 // Clock Initialization
    GPIO_Init();                // GPIO ports Initialization
    ADC_Init();                 // ADC Initialization
    TIMERB_Init();              // Timer B0 Initialization for Stall fault detection and Fault recovery time
    TimerA0_Init();             // Timer A0 Initialization to generate 4 PWM's for switches
    TimerA2_Init();             // Timer A2 Initialization to generate 2 PWM's for A phase

    __bis_SR_register(GIE);     /*enable global interrupt */
}

void DRV8x_Analog_Init(void)
{
    if(sensoredTrapController.SPIVariant == TRUE)                  // Initialize the SPI variables and settings only if the device is "S" Variant
    {
    	SPI_Init();             // Initialize EVM8343 SPI in slave mode
    	drv83xx_Control_Init(); // Initialize the computational values for register settings
    	drv83xx_Register_Read();
    	drv83xx_Register_Init();
    	drv83xx_Register_Write();
    }
}


void HostControllerInit(void)
{
	HostController.EnabledGateDrivers = GATE_DISABLED;
	HostController.StartStopMotor = STOP_MOTOR;

	HostControl_Status = HOST_IDLE;
}
