/**************************************************************************
 * @file        global.c
 * @author      MDBU Software Team
 * @brief       This file defines all the functions used by application during runtime
 * @note        Copyright (c) 2016 Texas Instruments Incorporated.
 *              All rights reserved.
 ******************************************************************************/

#ifndef global_H_
#define global_H_

#include "Init.h"
#include "dataTypeDefinition.h"
#include "msp430.h"
#include "SPI_API.h"
#include "TrapSensored_Parameters_Setup.h"
#include "mdbuserial.h"
#include "mdbu_global.h"
#include "mdbuserial.h"
#include "drv8x_HAL.h"

// Motor Parameter Numbers
#define MTR_PARAM_DIR	(14)
#define MTR_PARAM_SPEED	(15)
#define MTR_PARAM_DEVICE_ID		(2)
#define MTR_MIN_DUTY_CYCLE		(4)
#define MTR_MAX_DUTY_CYCLE		(5)
#define MTR_RAMP_RATE_DELAY		(6)
#define MTR_UNDER_VOLTAGE_LIMIT		(7)
#define MTR_OVER_VOLTAGE_LIMIT		(8)
#define MTR_MIN_STALLDETECT_DUTY	(9)
#define MTR_STALLDETECT_REV_THRESHOLD	(10)
#define MTR_STALLDETECT_TIMER_THRESHOLD	(11)
#define MTR_AUTO_FAULT_RECOVERY_TIME	(12)
#define MTR_MOTOR_PHASE_CURRENT_LIMIT	(13)
#define MTR_FAULT_STATUS				(1)
#define MTR_PARAM_ELEC_SPEED 			(0)
#define MTR_PARAM_PWM_FREQ				(3)
#define MTR_START_STOP_MOTOR			(16)
#define MTR_HALL_CALIBRATE  			(17)
#define MTR_CALIBRATE_DUTY_CYCLE		(18)
#define MTR_CALIBRATION_CYCLES		    (19)
#define MTR_AUTO_CALIBRATION            (20)
#define DEVICE_FULL_SCALE_VOLTAGE       (21)
#define DEVICE_CLEAR_FAULTS             (45)

#define EN_DRV	    BIT6    // P1.6
#define nFAULT	    BIT7    // P2.7

#define DRV8304_DEVICE							3      // Device variant used for DRV832x

#define HALL_PIN_A_BIT_POS 					    6      // Port Bit value where Hall A is connected. P2.6
#define HALL_PIN_B_BIT_POS 					    3	   // Port Bit value where Hall B is connected. P2.3
#define HALL_PIN_C_BIT_POS 					    0      // Port Bit value where Hall C is connected. P2.0


#define HALL_SHIFT_FOR_PIN_A 					(HALL_PIN_A_BIT_POS - 0)      // Bit shifting needed to bring Hall 'A' to position "0" from above values
#define HALL_SHIFT_FOR_PIN_B 					(HALL_PIN_B_BIT_POS - 1)      // Bit Shifting needed to bring Hall 'B' to position "1" from above values
#define HALL_SHIFT_FOR_PIN_C 					(HALL_PIN_C_BIT_POS + 2)      // Bit Shifting needed to bring Hall 'C' to position "2" from above values

#define SIX_PWM_MODE 							0     // 6x PWM mode selection value
#define ONE_PWM_MODE 							1     // 1x PWM mode selection value
#define START_MOTOR							    0     // Indicates Motor is Started
#define STOP_MOTOR							    1     // Indicates Motor is in Stop state
#define GATE_DISABLED							0     // Indicates Gate Driver enable
#define GATE_ENABLED							1     // Indicates Gate Driver enable

/* Device register width in number of bytes */
#define drv83xx_REGISTER_WIDTH (2)

void EnableGateDrivers();
void DisableGateDrivers();
void SetPWMDutyCycle(uint16_t PWMDutyCycle);
void UpdateNextCommutation(void);
void Hall_AlignSetState(uint8_t hallState);
void PWM_SetCommutation(uint8_t hallState);
void ReadVCC(void);
void ReadCurrentShunt(void);
void ReadSPDFDBK(void);

void drv83xx_setGPIO(unsigned char gpioPort, unsigned char gpioNum, unsigned char gpioVal);
unsigned char drv83xx_getGPIO(unsigned char gpioPort, unsigned char gpioNum);
void drv83xx_StartMotor();
void drv83xx_StopMotor();
unsigned int drv83xx_registerRead(unsigned char address);
void drv83xx_registerWrite(unsigned char address, unsigned int value);
unsigned long drv83xx_getMtrParam(unsigned char num);
void drv83xx_setMtrParam(unsigned char num, unsigned long value);
void drv83xx_setCtrlType(unsigned char value);
unsigned char drv83xx_getCtrlType(void);


typedef enum
{
    HOST_EN = 0,
    HOST_IDLE = 1,
    HOST_ACTIVE = 2
} HOSTCONTROL_STATUS;

typedef struct HOST_CONTROLLER_Obj
{
	uint8_t EnabledGateDrivers;		/* 0 = Disabled, 1 = Enabled */
	uint8_t StartStopMotor;			/* 0 = Start, 1 = Stop */
	uint8_t CalibrateHall;          /* 1 = Start, 0 = Stop */
} HOST_CONTROLLER_Obj;

typedef enum
{
    SYSTEM_INIT = 0,
    SYSTEM_IDLE = 1,
    MOTOR_IDLE = 2,
    MOTOR_RAMP_UP = 3,
    MOTOR_RUN = 4,
    MOTOR_RAMP_DOWN = 5,
    MOTOR_STOP = 6,
    FAULT = 7,
    MOTOR_DIRECTION = 8,
	HALL_ALIGN = 9
}STATE_MACHINE;

typedef enum
{
    NOFAULT = 0,
	VOLTAGE = 1,
	OVERCURRENT = 2,
	OVERTEMPERATURE = 3,
	MOTOR_STALL = 4,
	HALLSENSORINVALID = 5,
	GATE_DRIVER = 6,
	POWER_SUPPLY = 7,
	UNKNOWN = 8,
	HALL_CALIBRATION =9

} FAULTS;

typedef struct APPLICATION_STATUS
{
    STATE_MACHINE currentstate;
    STATE_MACHINE previousstate;
    FAULTS fault;
} APPLICATION_STATUS;

typedef struct SENSORED_TRAP_Obj
{
    BOOL HallAlignStartFlag;     /* Flag to indicate the start aligning the rotor to Hall identification vector */
    BOOL HallCalibComplete;      /* Flag to indicate the end of calibration of Hall Signals */
    BOOL AutoHallCalib;			 /* Flag to indicate the auto calibration of Hall signals */
    BOOL HallAlignDoneFlag;      /* Flag to indicate the end of aligning the rotor to Hall identification vector */
    BOOL HallAlignInterruptFlag; /* Falg to indicate for verification of Hall alignment */
    BOOL TimerOverflowFlag;  		 /* Interrupt counter to ensure the speed measurement with 16bit resolution*/
    BOOL GateToggleFaultDetect;      /* For DRV83xxH devices Upon Enabling gate drivers, reporting of faults is disabled for first 5 ms, reporting is enabled after 10ms */

    uint8_t HallAlignState;		/* Indicates the Hall align state */
    uint8_t currentHallstate;	/* Indicates the Current Hall align state */
    uint8_t ShiftForPINA;       /* Indicates the Shift required for HallA read status proper positioning according to MCU */
    uint8_t ShiftForPINB;		/* Indicates the Shift required for HallB read status proper positioning according to MCU */
    uint8_t ShiftForPINC;		/* Indicates the Shift required for HallC read status proper positioning according to MCU */
    uint8_t ADCReadState;      /* specifies in which state ADC read has been initaiated */
    uint8_t Direction;         /* specifies the current direction of rotation */
    uint8_t Direction_flag;    /* flag to monitor if direction change is required */

    uint16_t IdriveP_Min_Value;  /* Indicates the IdriveP minimum value */
    uint16_t IdriveN_Min_Value;  /* Indicates the IdriveN minimum value */
    uint16_t Tdrive_Max_Value;   /* Indicates the IdriveP maximum value */
    uint16_t IdriveP_Setting;    /* Indicates the IdriveN maximum value */
    uint16_t IdriveN_Setting;    /* Indicates the IdriveP Setting value */
    uint16_t Tdrive_Setting;     /* Indicates the IdriveP Setting value */

    uint16_t Accelerationcounter;     /*Acceleration variables Initialize a counter to compare with Acceleration Divider.*/
    uint16_t HallAligncounter;        /*Initialize a counter to compare with HallAlignWaitTime */
    uint16_t HallAlignValue[4];
    uint16_t Accelerationdivider;	    /*How many PWM cycles before increasing duty cycle. */
    uint16_t HallCalibrationCycles;		/* Defines number of PWM cycles Motor Hall Calibration is Executed */
    uint16_t MINDutyCycle;              /* Low speed setting depending on application*/
    uint16_t MAXDutyCycle;              /* High speed setting depending on application*/
    uint16_t CALDutyCycle;              /* Calibration duty cycle applied to motor during Hall Calibration */
    uint16_t RotationCount;				/*This keeps track of how many rotations (electrical) of the motor is completed since the last startup.*/
    uint16_t TargetDutyCycle;			/*Duty cycle that should be reached for the selected potentiometer command setting.*/

    uint16_t CurrentDutyCycle;			    /*Duty cycle that is going out on the PWM at this moment.*/
    uint16_t RampRate;                          /*How fast increasing and decreasing duty cycle*/

    /* Fault handling variables */
    uint16_t StallDetectCounter;        /*This is a counter used in Timer4 to check for the motor stall.*/
    uint16_t RestartDelay;              /*This is the variable that is used to count the time from when a fault is cleared*/
    uint16_t VCCvoltage;                /*How much voltage applied for EVM*/
    uint16_t MotorPhaseCurrent;         /* How much current flowing through motor phase */
    uint16_t underVoltageLimit;         /*Configurable Under Voltage Limit from HOST*/
    uint16_t overVoltageLimit;          /*Configurable Over Voltage Limit from HOST*/
    uint16_t minStallDetectDuty; 	    /* Minimum Duty Cycle above which stall will be detected */
    uint16_t stallDetectRevThreshold;   /* Number of revolutions below which stall fault will be flagged */
    uint16_t stallDetectTimerThreshold; /* Time in milli seconds above which if motor doesnt spin min revolutions specified above(stallDetectRevThreshold) a stall fault is triggered */
    uint16_t autoFaultRecoveryTime; 	/* Time in milli seconds after which system reinitialises itself if fault gets cleared */
    uint16_t readVccCounter;			/* Counter to read VCC every READ_VCC_PERIOD interval */
    uint16_t SPDFdbk;           		/* This variable holds the timer counts with 16bit resolution */
    uint16_t PWM_Mode;			  		/* This Variable holds the Ctrl type : 0 for 6PWM mode , 1 for 1 PWM mode*/
    uint16_t PWMPeriod;          		/* This Variable holds the Motor parameter to set the PWM switching frequency*/
    uint16_t DeviceID;         			/* This variable holds the Motor parameter (2) Device ID to set the appropriate motor control page for a connected device */
    uint16_t DeviceVariant;    			/* This variable holds the Device variant, Device variant is set to 1 for DRV835x devices and set to zero for drv83xx Variants */
    uint16_t ShuntVariant;     			/* This variable holds the Shunt variant, Shunt variant is set to 1 for DRV83x3x devices and set to zero for DRV83x0 Variants */
    uint16_t SPIVariant;       			/* This variable holds the SPI variant, SPI variant is set to 1 for drv83xxS devices and set to zero for drv83xxH Variants */
    uint16_t MinPowerSupply;   			/* This variable holds the Digital equivalent of 5v */
    uint16_t PowerSupplyRecovery;   			/* This variable holds the power supply recovery fault status */
    uint16_t GateToggleCounter;         /* This counter is used to count for first 5 ms after changing gate drive status to toggle Gate drive fault detect */
    uint16_t Counter_1M_Second;

    float_t MotorPhaseCurrentLimit;     /* Defines Maximum allowed Motor phase current  */

} SENSORED_TRAP_Obj;


/////////////////////////
#endif /*global_H_*/
